import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'base_view_state.dart';

mixin BaseMixRequestModelController on BaseViewState{

  List<RequestInfo> getRequestInfoList();

  List<HttpData> httpDataList(){
    List<HttpData> httpDataList = [];
    _httpDataCollection.forEach((key, value) {
      if(isEmpty(value.httpError)){
        httpDataList.add(value);
      }
    });
    return httpDataList;
  }

  Map<String, HttpData> _httpDataCollection = {};


  void refreshData(){

    List<RequestInfo> requestInfoList = getRequestInfoList();

    if(isNotEmpty(requestInfoList)){
      requestInfoList.forEach((requestInfo) async{
        if(isSync){
          ///同步请求
          HttpData httpData = await doRequest(requestInfo);
          _httpDataCollection[requestInfo.url] = httpData;
          _httpDataCollection.removeWhere((key, value) => !value.success);
          onCompleted(_httpDataCollection);
        }else{
          ///异步请求
          doRequest(requestInfo).then((httpData){
            _httpDataCollection[requestInfo.url] = httpData;
            _httpDataCollection.removeWhere((key, value) => !value.success);
            onCompleted(_httpDataCollection);
            setIdle();

            if(_httpDataCollection.length == requestInfoList.length){
              bool allError = true;
              List<HttpErrorType> httpErrorTypeList = [];
              _httpDataCollection.forEach((key, value){
                if(isEmpty(value.httpError)){
                  allError = false;
                }else{
                  httpErrorTypeList.add(value.httpError.httpErrorType);
                }
              });
              if(allError){
                HttpError httpError = HttpError(HttpErrorType.Unknown, HTTP_ERROR_MSG, logMsg: '异步多个请求失败： $httpErrorTypeList');
                setError(httpError);
              }
              refreshController.finishRefresh(success: !allError);
            }
          });
        }
      });
    }

    if(isSync){
      bool allError = true;
      List<HttpErrorType> httpErrorTypeList = [];
      _httpDataCollection.forEach((key, value){
        if(isEmpty(value.httpError)){
          allError = false;
        }else{
          httpErrorTypeList.add(value.httpError.httpErrorType);
        }
      });
      if(allError){
        HttpError httpError = HttpError(HttpErrorType.Unknown, HTTP_ERROR_MSG, logMsg: '同步多个请求失败： $httpErrorTypeList');
        setError(httpError);
      }else{
        setIdle();
      }
      refreshController.finishRefresh(success: !allError);
    }
  }


  HttpData getHttpDataByUrl(String url){
    if(_httpDataCollection.containsKey(url)){
      return _httpDataCollection[url];
    }
    return null;
  }


  onCompleted(Map<String, HttpData> httpDataCollection) {}


  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      refreshData();
    }
  }


  ///默认异步请求
  bool get isSync => false;

  @override
  RequestInfo getRequestInfo() {
    return null;
  }


  ///获取一个公共的刷新控件
  Widget getEasyRefreshWidget({
    ScrollController scrollController,
    Widget footWidget,
    Widget emptyWidget,
    ViewStateModelWidgetListBuilder<Map<String, HttpData>> widgetListBuilder,
    ViewStateModelWidgetBuilder<Map<String, HttpData>> widgetBuilder,
    Widget busyWidget,
    bool showErrorWidget = true,
    bool showEmptyWidget = true,
    bool showBusyWidget = true,
    Function onLoad,
    Function onRefresh,
  }){
    return buildViewStateEasyRefresh<Map<String, HttpData>>(
        data: _httpDataCollection,
        scrollController: scrollController,
        footWidget: footWidget,
        emptyWidget: emptyWidget,
        widgetListBuilder: widgetListBuilder,
        widgetBuilder: widgetBuilder,
        busyWidget: busyWidget,
        showBusyWidget: showBusyWidget,
        showEmptyWidget: showEmptyWidget,
        showErrorWidget: showErrorWidget,
        onLoad: onLoad,
        onRefresh: onRefresh??(){
          refreshData();
        }
    );
  }

}